<?php

namespace OCA\GrueneGroupfolderManagement\Controller;

use Exception;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\Attribute\NoAdminRequired;
use OCP\App\IAppManager;
use Psr\Container\ContainerInterface;

use OCA\GroupfolderFilesystemSnapshots\Manager\SnapshotManager;
use OCA\GroupfolderFilesystemSnapshots\Service\DiffTaskService;
use OCA\GroupfolderFilesystemSnapshots\Service\DiffTaskResultService;
use OCA\GroupfolderFilesystemSnapshots\Db\DiffTask;

use OCA\GrueneGroupfolderManagement\Db\GroupMapper;
use OCA\GrueneGroupfolderManagement\Exceptions\NotFoundException;
use OCA\GrueneGroupfolderManagement\Http\StreamedProgressResponse;
use OCA\GrueneGroupfolderManagement\Manager\GroupFolderManager;
use OCA\GrueneGroupfolderManagement\Manager\GroupManager;
use OCA\GrueneGroupfolderManagement\Groups\GroupName;

class DiffController extends BaseController {

	private bool $snapshotIntegrationEnabled;
	private SnapshotManager $snapshotManager;
	private DiffTaskService $diffTaskService;
	private DiffTaskResultService $diffTaskResultService;

	public function __construct(
		private IAppManager $appManager,
		private ContainerInterface $container,

        private GroupFolderManager $groupFolderManager,
        private GroupManager $groupManager,
		private GroupMapper $groupMapper,
        private ?string $userId,
	) {
		parent::__construct();

		$this->snapshotIntegrationEnabled = $this->appManager->isEnabledForUser("groupfolder_filesystem_snapshots");

		if($this->snapshotIntegrationEnabled) {
			$this->snapshotManager = $this->container->get(SnapshotManager::class);
			$this->diffTaskService = $this->container->get(DiffTaskService::class);
			$this->diffTaskResultService = $this->container->get(DiffTaskResultService::class);
		}
	}

    use Errors;

    #[NoAdminRequired]
    public function create(string $snapshotId, int $groupId, bool $streamed = false, bool $includeResults = false) {
		if(!$this->snapshotIntegrationEnabled) {
			return new JSONResponse([
				"status" => "error",
				"errorMessage" => "Snapshot integration not active",
			], Http::STATUS_NOT_FOUND);
		}

        $group = $this->groupManager->get($groupId);

		$this->denyAccessUnlessGranted(['UPDATE'], $group);

        $divisionGroupFolderId = (int)($this->groupFolderManager->getGroupFolderByOrganizationKey($group->getDivisionKey())['folder_id']);
        //$rootGroupFolder = $this->groupFolderManager->getGroupFolder($rootGroupFolderId, false);

        $groupFolderRelativePath = GroupName::cleanName($group->getName());
        //$groupFolder = $rootGroupFolder->get($directoryName);
        //$groupFolderRelativePath = $groupFolder->getInternalPath();

        if(!$this->snapshotManager->snapshotExists($snapshotId)) {
            throw new NotFoundException(SnapShotManager::class, ['id' => $snapshotId]);
        }
        
        //return new JSONResponse($groupFolderRelativePath);

        if($streamed) {
            return new StreamedProgressResponse(function() use ($groupFolderRelativePath, $divisionGroupFolderId, $snapshotId, $includeResults) {
                try {
                    $this->diffTaskService->create($groupFolderRelativePath, $divisionGroupFolderId, $snapshotId, $this->userId, function(array $progress) use ($includeResults) {
                        if($includeResults && isset($progress["result"])) {
                            $progress["result"] = $this->addResultsToDiffTask($progress["result"]);
                            //throw new Exception('Test Exception');
                        }
                        
                        echo json_encode($progress) . "\n";
                    });
                } catch(Exception $e) {
                    echo json_encode([
                        "status" => "error",
                        "errorMessage" => $e->getMessage(),
                    ]) . "\n";
                }
                
            });
        } else {
            $task = $this->diffTaskService->create($groupFolderRelativePath, $groupId, $snapshotId, $this->userId);

            if(!isset($task)) {
                return new JSONResponse([
                    "status" => "error",
                    "errorMessage" => "Unknown Error",
                ], Http::STATUS_NOT_FOUND);
            }

            if($includeResults) {
                $task = $this->addResultsToDiffTask($task);
            }
            
            return new JSONResponse($task);
        }
    }

    #[NoAdminRequired]
    public function show(string $snapshotId, int $diffTaskId): JSONResponse {
		if(!$this->snapshotIntegrationEnabled) {
			return new JSONResponse([
				"status" => "error",
				"errorMessage" => "Snapshot integration not active",
			], Http::STATUS_NOT_FOUND);
		}

        $task = $this->diffTaskService->find($diffTaskId, $this->userId);

		$this->denyAccessUnlessGranted(['READ'], $task);

        if($task->getSnapshotId() == $snapshotId) {
            return new JSONResponse($task);
        } else {
            throw new NotFoundException(SnapShotManager::class, ['snapshotId' => $snapshotId, 'diffTaskId' => $diffTaskId]);
        }
    }

    private function addResultsToDiffTask(DiffTask $task): Array {
        $taskArray = $task->jsonSerialize();
        $taskArray["results"] = $this->diffTaskResultService->findAll($taskArray["id"]);
        return $taskArray;
    }
}